import { Observable } from 'rxjs';
import { AsyncValidator, AsyncValidatorFn, ValidationErrors, Validator, ValidatorFn } from './core/validators';
import { AbstractControl } from './model';
export declare class Validators {
    static min(min: number): ValidatorFn;
    static max(max: number): ValidatorFn;
    static required(control: AbstractControl): ValidationErrors | null;
    static requiredTrue(control: AbstractControl): ValidationErrors | null;
    static email(control: AbstractControl): ValidationErrors | null;
    static minLength(minLength: number): ValidatorFn;
    static maxLength(maxLength: number): ValidatorFn;
    static pattern(pattern: string | RegExp): ValidatorFn;
    static nullValidator(control: AbstractControl): ValidationErrors | null;
    static compose(validators: null): null;
    static compose(validators: (ValidatorFn | null | undefined)[]): ValidatorFn | null;
    static composeAsync(validators: (AsyncValidatorFn | null)[]): AsyncValidatorFn | null;
}
export declare function minValidator(min: number): ValidatorFn;
export declare function maxValidator(max: number): ValidatorFn;
export declare function requiredValidator(control: AbstractControl): ValidationErrors | null;
export declare function requiredTrueValidator(control: AbstractControl): ValidationErrors | null;
export declare function emailValidator(control: AbstractControl): ValidationErrors | null;
export declare function minLengthValidator(minLength: number): ValidatorFn;
/**
 * Validator that requires the length of the control's value to be less than or equal
 * to the provided maximum length. See `Validators.maxLength` for additional information.
 */
export declare function maxLengthValidator(maxLength: number): ValidatorFn;
/**
 * Validator that requires the control's value to match a regex pattern.
 * See `Validators.pattern` for additional information.
 */
export declare function patternValidator(pattern: string | RegExp): ValidatorFn;
/**
 * Function that has `ValidatorFn` shape, but performs no operation.
 */
export declare function nullValidator(control: AbstractControl): ValidationErrors | null;
export declare function toObservable(r: any): Observable<any>;
/**
 * Given the list of validators that may contain both functions as well as classes, return the list
 * of validator functions (convert validator classes into validator functions). This is needed to
 * have consistent structure in validators list before composing them.
 *
 * @param validators The set of validators that may contain validators both in plain function form
 *     as well as represented as a validator class.
 */
export declare function normalizeValidators<V>(validators: (V | Validator | AsyncValidator)[]): V[];
/**
 * Accepts a list of validators of different possible shapes (`Validator` and `ValidatorFn`),
 * normalizes the list (converts everything to `ValidatorFn`) and merges them into a single
 * validator function.
 */
export declare function composeValidators(validators: Array<Validator | ValidatorFn>): ValidatorFn | null;
/**
 * Accepts a list of async validators of different possible shapes (`AsyncValidator` and
 * `AsyncValidatorFn`), normalizes the list (converts everything to `AsyncValidatorFn`) and merges
 * them into a single validator function.
 */
export declare function composeAsyncValidators(validators: Array<AsyncValidator | AsyncValidatorFn>): AsyncValidatorFn | null;
/**
 * Merges raw control validators with a given directive validator and returns the combined list of
 * validators as an array.
 */
export declare function mergeValidators<V>(controlValidators: V | V[] | null, dirValidator: V): V[];
/**
 * Retrieves the list of raw synchronous validators attached to a given control.
 */
export declare function getControlValidators(control: AbstractControl): ValidatorFn | ValidatorFn[] | null;
/**
 * Retrieves the list of raw asynchronous validators attached to a given control.
 */
export declare function getControlAsyncValidators(control: AbstractControl): AsyncValidatorFn | AsyncValidatorFn[] | null;
/**
 * Accepts a singleton validator, an array, or null, and returns an array type with the provided
 * validators.
 *
 * @param validators A validator, validators, or null.
 * @returns A validators array.
 */
export declare function makeValidatorsArray<T extends ValidatorFn | AsyncValidatorFn>(validators: T | T[] | null): T[];
/**
 * Determines whether a validator or validators array has a given validator.
 *
 * @param validators The validator or validators to compare against.
 * @param validator The validator to check.
 * @returns Whether the validator is present.
 */
export declare function hasValidator<T extends ValidatorFn | AsyncValidatorFn>(validators: T | T[] | null, validator: T): boolean;
/**
 * Combines two arrays of validators into one. If duplicates are provided, only one will be added.
 *
 * @param validators The new validators.
 * @param currentValidators The base array of currrent validators.
 * @returns An array of validators.
 */
export declare function addValidators<T extends ValidatorFn | AsyncValidatorFn>(validators: T | T[], currentValidators: T | T[] | null): T[];
export declare function removeValidators<T extends ValidatorFn | AsyncValidatorFn>(validators: T | T[], currentValidators: T | T[] | null): T[];
